/* A simple app to show CPU usage in percent 
by Brent Miszalski (misza@ihug.com.au)
You may use this code as you wish :-)
*/
#include <iostream>
#include <OS.h>
#include <MessageRunner.h>
#include <Application.h>
#include <stdio.h>
#include <String.h>
const char* APP_SIG = "application/x-vnd.misza-cpu_percentage";
const uint32 STARTME = 'Init';
const int UPDATE_TIME = 500000;	//Set how fast it updates here... (in µs)
class cpu_percent_app : public BApplication
{
	public:
		cpu_percent_app();
		virtual void MessageReceived(BMessage* msg);
		void UpdateCPUPercent();
	private:
		double last_active_cpu_time,last_active_cpu_time2;
		double delta_cpu, delta_cpu2;
		double last_system_time,delta_system;
		double percent,percent2;
		double lastpercent,lastpercent2;
		BMessageRunner* update;
		BString cpu1,cpu2;
		bool Is2CPU;
};

cpu_percent_app::cpu_percent_app() : BApplication(APP_SIG)
{
	system_info info;
	get_system_info(&info);
	if(info.cpu_count >=2)
		Is2CPU = true;
	else
		Is2CPU = false;
	UpdateCPUPercent();
	
	
	BMessenger mymsnger(this);
	BMessage update_ram(STARTME);
	//sends lots of BMessages (STARTME) (1 every UPDATE_TIME µs) infinitely (-1)
	update = new BMessageRunner(mymsnger,&update_ram,UPDATE_TIME,-1);


}

void cpu_percent_app::UpdateCPUPercent()
{
	system_info info;
	get_system_info(&info);
	
	delta_system = (double)system_time() - last_system_time;
	if(!Is2CPU)
	{
	//cpu 1
	delta_cpu = double(info.cpu_infos[0].active_time) - last_active_cpu_time;
	
	percent = (delta_cpu/delta_system);
	if(percent < 0) percent = 0.00;
	if(percent > 1) percent = 1.00;
	cpu1 = "";
	cpu1 << (float)percent*100 << "%";
	lastpercent = percent;
	last_active_cpu_time = (double)info.cpu_infos[0].active_time;
	}
	if(Is2CPU)
	{
	//cpu 1
	delta_cpu = double(info.cpu_infos[0].active_time) - last_active_cpu_time;
	
	percent = (delta_cpu/delta_system);
	if(percent < 0) percent = 0.00;
	if(percent > 1) percent = 1.00;
	cpu1 = "";
	cpu1 << (float)percent*100 << "%";
	lastpercent = percent;
	last_active_cpu_time = (double)info.cpu_infos[0].active_time;
	
	//cpu 2
	delta_cpu2 = double(info.cpu_infos[1].active_time) - last_active_cpu_time2;
	
	percent2 = (delta_cpu2/delta_system);
	if(percent2 < 0) percent2 = 0.00;
	if(percent2 > 1) percent2 = 1.00;
	
	cpu2 = "";
	cpu2 << (float)percent2*100 << "%";
	last_active_cpu_time2 = (double)info.cpu_infos[1].active_time;
	lastpercent2 = percent2;
	}
	last_system_time = (double)system_time();
}

void cpu_percent_app::MessageReceived(BMessage* msg)
{
	switch(msg->what)
	{
		case STARTME:
		{
			UpdateCPUPercent();
			if(Is2CPU)
				cout << "CPU 1: " << cpu1.String() << " CPU 2: " << cpu2.String() << endl;
			else
				cout << "CPU 1: " << cpu1.String() << endl;
		}break;
	
		default:
			BApplication::MessageReceived(msg);
		break;
	}
}
int main(int argc, char ** argv)
{
	BApplication* app = new cpu_percent_app();
	app->Run();
	return 0;
}